Rem
	Shadow Demo
	
	Copyright:
		- TProgramObject: blitzbasic.com
		- TImageBuffer: blitzforum.de - klepto2
EndRem

SuperStrict

Global clog:Byte[1024*4]


Type TVector
	Field X:Float, Y:Float
	
	Function Create:TVector(X:Float = 0, Y:Float = 0)
		Local Vec:TVector = New TVector
		Vec.X = X
		Vec.Y = Y
		
		Return Vec
	EndFunction
	
	Method Normalize:TVector()
		Return TVector.Create(-Y,X)
	EndMethod
	
	Method Length:Float()
		Return Sqr(x*x + y*y)
	EndMethod
	
	Method Mul(Value:Float)
		X:*Value
		Y:*Value
	EndMethod
	Method Div(Value:Float)
		X:/Value
		Y:/Value
	EndMethod
	Method Add(VEc:TVector)
		X:+Vec.X
		Y:+Vec.Y
	EndMethod
EndType

Type TColor
	Field R:Float, G:Float, B:Float, A:Float
	
	Function Create:TColor(R:Byte, G:Byte, B:Byte, A:Byte)
		Local COl:TColor = New TColor
		Col.R = r/255
		Col.G = g/255
		Col.B = b/255
		Col.A = A/255
		Return col
	EndFunction
EndType

Type TDrawable Abstract
	Method Update() Abstract
	Method Render() Abstract
EndType


Type TLight Extends TDrawable
	Global Lights:TList = New TList
	Global ShadowBuffer:TImageBuffer
	
	Field Position:TVector
	Field Radius:Float
	Field Color:TColor
	
	Method Update()
	
	EndMethod
		
	Method Render()
		LightProgram.Activate
		LightProgram.setUF2 "lightPosition", Position.X, GraphicsHeight()-Position.Y
		LightProgram.setUF1 "radius", Radius
		LightProgram.setUF4 "color", Color.R, Color.G, Color.B, Color.A
		
		glBegin GL_QUADS
		glVertex2f Position.X-Radius, Position.Y-Radius
		glVertex2f Position.X+Radius, Position.Y-Radius
		glVertex2f Position.X+Radius, Position.Y+Radius
		glVertex2f Position.X-Radius, Position.Y+Radius
		glEnd
		
		LightProgram.DeActivate
	EndMethod
	
	Method RenderShadow()
		'Schatten Rendern
		For Local Obj:TShadowEmitting=EachIn TShadowEmitting.ShadowEmitting
			Obj.RenderShadow(Self)
		Next
	EndMethod

	Function Create:TLight(Position:TVector, Radius:Float, Color:TColor = Null)
		If Not Color Color = TColor.Create(255,255,255,255)
		Assert Color
		Assert Position
		
		Local Light:Tlight = New TLight
		light.Position = Position
		light.Color = Color
		light.Radius = Radius
		
		Lights.AddLast Light
		Return light
	EndFunction
	
	Function RenderAll()
		'Shadow in den imagebuffer rendern
		For Local Light:TLight =EachIn Lights
			ShadowBuffer.Bind
			ShadowBuffer.Clear
			
			Light.RenderShadow
			
			ShadowBuffer.UnBind
			
			'Lights rendern \o/
			
			Light.Render
			
			ShadowBuffer.Draw
			
			
			LightAttentuation.Activate
			LightAttentuation.setUF2 "lightPosition", Light.Position.X, GraphicsHeight()-Light.Position.Y
			LightAttentuation.setUF1 "radius", Light.Radius
			TShadowEmitting.RenderAll()
			LightAttentuation.DeActivate
			
			ShadowBuffer.Draw
		Next
	EndFunction
EndType

Type TShadowEmitting Extends TDrawable
	Global ShadowEmitting:TList = New TList
	Const SHadowAlpha:Float = 1
	
	Field UsePoly:Byte = False
	Field Position:TVector
	Field Vectors:TVector[]
	Field Color:TColor
	
	Method GetShape:TVector[]()
		Return Vectors
	EndMethod
	
	Function RenderAll()
		For Local Obj:TShadowEmitting=EachIn ShadowEmitting
			Obj.Render
		Next
	EndFunction
	
	Method SetPosition(Pos:TVector)
	EndMethod
	
	Method Render()
		If Not UsePoly
			glBegin GL_Quads
		Else
			glBegin GL_POLYGON
		EndIf
		glColor4f Color.R, Color.G, Color.B, Color.A
		For Local Vector:TVector = EachIn Vectors
			glVertex2f Vector.X, Vector.Y
		Next
		glEnd
	EndMethod
	
	Method RenderShadow(Light:TLight)
		Local vs:TList = New TList
		glBegin GL_TRIANGLE_strip
		glColor4f 0,0,0,ShadowAlpha
		
		
		Local OldSP:Int = 0
		'nun crazy funky action
		Local MakeShadow:Byte = False
		Local PrevVec:TVector
		For Local i:Int = 0 To Vectors.Length*2
			Local CurVec:TVector = Vectors[i Mod Vectors.Length]
			Local SuccVec:TVector = Vectors[(i + 1) Mod Vectors.Length]
			Local CurSP:Int = (CurVec.Y - SuccVec.Y)*(CurVec.X - Light.Position.X) + (SuccVec.X - CurVec.X)*(CurVec.Y - Light.Position.Y)
			
			If MakeShadow And CurSP < 0
				Local v:Tvector[] = [curvec, succvec]
				vs.AddLast v
				
				DoVertex Light, CurVec, SuccVec
			EndIf
			
			
			If i > 0
				If CurSP =< 0 And OldSP > 0
					
					MakeShadow = True
					
					Local v:Tvector[] = [curvec, succvec]
					vs.AddLast v
					
					glVertex2f CurVec.X, CurVec.Y
					DoVertex Light, CurVec, SuccVec
				ElseIf CurSP >= 0 And OldSP < 0 And MakeShadow
					MakeShadow = False
					If PrevVec DoVertex Light, CurVec, PrevVec
					Exit
				EndIf
			EndIf
			
			
			
			OldSP = CurSP
			PrevVec = CurVec
		Next
		glEnd
	EndMethod
	
	Method DoVertex(Light:TLight, CurVec:TVector, SuccVec:TVector)
		glVertex2f curvec.X + (CurVec.X - Light.Position.X)*100, CurVec.Y + (CurVec.Y - Light.Position.Y)*100
		glVertex2f succVec.X, succVec.Y
	EndMethod
	
	Method update()
	EndMethod
EndType

Type TRectangle Extends TShadowEmitting
	Field Width:Int, Height:Int
	Function Create:TRectangle(Position:TVector, Width:Float, Height:Float, Color:TColor = Null)
		If Not Color Color = TColor.Create(255,255,255,255)
		
		Assert Position
		Assert Width > 0
		Assert Height > 0
		
		Local Rect:TRectangle = New TRectangle
		Rect.Color = Color
		Rect.SetPosition Position
		rect.Width = Width
		Rect.Height = Height
		ShadowEmitting.AddLast Rect
		
		Return Rect
	EndFunction
	
	Method SetPosition(Pos:TVector)
		Self.Position = Pos
		Vectors	 = 	[TVector.Create(Position.X, Position.Y), ..
					 TVector.Create(Position.X + Width, Position.Y), ..
					 TVector.Create(Position.X + Width, Position.Y + Height), ..
					 TVector.Create(Position.X, Position.Y + Height)]
	EndMethod
EndType

Type TOval Extends TShadowEmitting
	Field Width:Int, Height:Int
	Function Create:TOval(Position:TVector, Width:Float, Height:Float, StepSize:Int = 8, Color:TColor = Null)
		If Not Color Color = TColor.Create(255,255,255,255)
		
		Assert Position
		Assert Width > 0
		Assert Height > 0
		Assert Stepsize > 0
		Width:/2
		Height:/2
		
		
		Local Oval:TOval = New TOval
		Oval.Position = Position
		Oval.Color = Color
		Oval.Width = Width*2
		Oval.Height = Height*2
		Oval.UsePoly = True
		For Local i:Int = 0 To 360 - StepSize
			Oval.Vectors :+ [TVector.Create(Position.X + Cos(i)*Width, Position.Y + Sin(i)*Height)]
			i:+StepSize-1
		Next
		ShadowEmitting.AddLast Oval
		
		Return Oval
	EndFunction
EndType

SetGraphicsDriver GLMax2DDriver()

Graphics(800, 600, 0, 60, GRAPHICS_BACKBUFFER|GRAPHICS_DEPTHBUFFER|GRAPHICS_STENCILBUFFER)
glewinit

'SetLineWidth 4


TLight.ShadowBuffer = TImageBuffer.Create(CreateImage(Pow2Size(GraphicsWidth()), Pow2Size(GraphicsHeight())))

Global LightAttentuation:TProgramObject = TProgramObject.Create("lightatt")
Global LightProgram:TProgramObject = TProgramObject.Create("light")

SetBlend AlphaBlend
Local light:TLight = TLight.Create(TVector.Create(100,100), 300,TColor.Create(255,255,0,255))

TRectangle.Create(TVector.Create(40,40), 80,60)
TRectangle.Create(TVector.Create(290,200), 40,80, TColor.Create(0,255,0,255))
Local R:TRectAngle = TRectangle.Create(TVector.Create(250,100), 40,80)

TOval.Create(TVector.Create(100,300), 60,20, 16)
TOval.Create(TVector.Create(150,505), 30, 30, 32)
TOval.Create(TVector.Create(600,450), 128, 300, 32)


Local start:Int = MilliSecs()
Repeat
	glClear GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT|GL_STENCIL_BUFFER_BIT
	
	R.Position.X:+(MilliSecs()-start)/10.0
	start = MilliSecs()
	If R.Position.X>GraphicsWidth() R.Position.X = -R.Width
	R.SetPosition TVector.Create(R.Position.X, R.Position.Y)
	
	TLight.RenderAll()
	
	
	light.Position.X = GraphicsWidth()/2 + Cos(MilliSecs()/10)*150 - 80; light.Position.Y = GraphicsHeight()/2 + Sin(MilliSecs()/10)*300
	DrawText "FPS: "+GetFPS(), 10, 10
	Flip 0
Until KeyHit(KEY_ESCAPE) Or AppTerminate()


Type TImageBuffer 
	Field Image:TImage 
	Field rb:Int[1] 
	Field fb:Int[1] 
	Field Imageframe:TGLImageframe 
	Field Frame:Int = 0 
	Field OrigX:Int 
	Field OrigY:Int 
	Field OrigW:Int 
	Field OrigH:Int 
	
	Function Create:TImageBuffer(Image:TImage,Frame:Int = 0 ) 
		Local IB:TImageBuffer = New TImageBuffer 
		IB.Image = Image 
		IB.Frame = Frame 
		IB.GenerateFBO() 
		
		Return IB 
	End Function 
	
	Method GenerateFBO() 
		ImageFrame = TGLImageFrame(Image.frame(Frame) ) 
		
		'imageframe.v0 = imageframe.v1 
		'imageframe.v1 = 0.0 
		
		Local W:Int = Image.width 
		Local H:Int = Image.Height 
		
		'AdjustTexSize(W , H) 
		
		
		glGenFramebuffersEXT(1, fb ) 
		glGenRenderbuffersEXT(1 , rb) 
		
		glBindTexture(GL_TEXTURE_2D, Imageframe.name); 
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT , fb[0]) ; 
		
		
		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D,  Imageframe.name, 0); 
		glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, rb[0]); 
		glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, W, H); 
		glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT , GL_DEPTH_ATTACHMENT_EXT , GL_RENDERBUFFER_EXT , rb[0]) 
		
		
		Local status:Int =  glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT) 
		
		Select status 
			Case GL_FRAMEBUFFER_COMPLETE_EXT 
				Print "all right" + " : " + Status 
			Case GL_FRAMEBUFFER_UNSUPPORTED_EXT 
				Print "choose different formats" 
			Default 
				End 
		EndSelect 
		
		'glBindTexture GL_TEXTURE_2D, 0
		glBindFrameBufferExt GL_FRAMEBUFFER_EXT, 0
	End Method 
	
	Method Bind() 
		GetViewport(OrigX,OrigY,OrigW,OrigH) 
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT , fb[0]) 
		SetViewport 0,0, Image.Width, Image.Height
	End Method 
	
	Method UnBind() 
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT , 0) 
		SetViewport 0,0,OrigW, OrigH
	End Method 
	
	Method Draw(X:Float = 0, Y:Float = 0)
		y:-GraphicsHeight()
		glEnable GL_TEXTURE_2D
		glBindTexture GL_TEXTURE_2D, Imageframe.Name

		glBegin GL_QUADS
		gltexcoord2f 0,0
		glVertex2f x,-y
		
		gltexcoord2f 1,0
		glVertex2f Image.Width+x,-y
		
		gltexcoord2f 1,1
		glVertex2f Image.Width+x,-(Image.Height+y)
		
		gltexcoord2f 0,1
		glVertex2f x,-(Image.Height+y)
		glEnd
		
		glBindTexture GL_TEXTURE_2D, 0
	EndMethod
	
	Method Clear()
		SetAlpha 0
		SetBlend SHADEBLEND
		DrawRect 0,0,Image.Width, image.height
		SetAlpha 1
		SetBlend ALPHABLEND
	End Method 
	
	Method BufferWidth:Int() 
		Return Image.Width 
	End Method 
	
	Method BufferHeight:Int() 
		Return Image.Height 
	End Method 
End Type 


Function AdjustTexSize( width:Int Var,height:Int Var ) 
   'calc texture size 
   width=Pow2Size( width ) 
   height=Pow2Size( height ) 
   Repeat 
      Local t:Int 
      glTexImage2D GL_PROXY_TEXTURE_2D,0,4,width,height,0,GL_RGBA,GL_UNSIGNED_BYTE,Null 
      glGetTexLevelParameteriv GL_PROXY_TEXTURE_2D,0,GL_TEXTURE_WIDTH,Varptr t 
      If t Return 
      If width=1 And height=1 RuntimeError "Unable to calculate tex size" 
      If width>1 width:/2 
      If height>1 height:/2 
   Forever 
End Function 

Function Pow2Size:Int( n:Int ) 
   Local t:Int=1 
   While t<n 
      t:*2 
   Wend 
   Return t 
End Function 



Function checkIfCompileError:String(id:Int)

	Local error:String = ""
	'Did the shader compile successfuly?
	Local compiled:Int
	glGetObjectParameterivARB(id,GL_OBJECT_COMPILE_STATUS_ARB, Varptr(compiled))
	If Not compiled
		Local logLength:Int
		glGetObjectParameterivARB(id,GL_OBJECT_INFO_LOG_LENGTH_ARB, Varptr(logLength))
		Local slen:Int = 0
		glGetInfoLogARB(id,1024,Varptr(slen), Varptr clog[0])
		Local err:String
		For Local i:Int = 0 To logLength-1
			err:+Chr(clog[i])
		Next
		error = err
	End If
	Return error
EndFunction

Function checkIfLinkError:String(Program:Int)
	Local linked:Int
	glGetObjectParameterivARB(Program,GL_OBJECT_LINK_STATUS_ARB, Varptr(linked))
	
	If Not linked
		'An Error occured during Linking
		
		Local slen:Int = 0

		glGetInfoLogARB(Program,1023,Varptr slen, Varptr clog[0])
		Local err:String
		Local i:Int
		For i = 0 To slen-1
			err:+Chr(clog[i])
		Next
		Return err
	End If
	Return ""
EndFunction

Type TProgramObject
	Field Program:Int
	
	Function Create:TProgramObject(Name:String)
		Local FragText:String, VertexText:String, FragLength:Int, VertLength:Int, VertCStr:Byte Ptr, FragCStr:Byte Ptr, Err:String
		FragText = LoadString("shader/"+Name+".frag")
		VertexText = LoadString("shader/"+Name+".vert")
		FragLength = FragText.Length
		VertLength = VertexText.Length
		VertCStr = VertexText.ToCstring()
		FragCStr = FragText.ToCString()
		
		
		Local ShaderProgram:Int = glCreateProgram()
		Local VertexShader:Int, FragmentShader:Int
		VErtexShader = glCreateShader(GL_VERTEX_SHADER)
		FragmentShader = glCreateShader(GL_FRAGMENT_SHADER)
		glShaderSource VertexShader, 1,Varptr VertCStr, Varptr VertLength
		glShaderSource FragmentShader, 1, Varptr FragCStr,Varptr FragLength
		glCompileShader VertexShader
		Err = checkIfCompileError(VErtexShader)
		If Err <> "" RuntimeError "VertexShader compile error: "+Err
		
		glCompileShader FragmentShader
		Err = checkIfCompileError(FragmentShader)
		If Err <> "" RuntimeError "FragmentShader compile error: "+Err
		
		glAttachShader ShaderProgram, VertexShader
		glAttachShader ShaderProgram, FragmentShader
		
		glDeleteShader VertexShader
		glDeleteShader FragmentShader
		MemFree VertCStr
		MemFree FragCStr
		
		glLinkProgram SHaderProgram
		Err = checkIfLinkError(ShaderProgram)
		
		If Err <> "" RuntimeError "Shader link error: "+Err
		
		Local programobj:TProgramObject = New TProgramObject
		programobj.Program = ShaderProgram
		Return programobj
	EndFunction
	
	
	Method Activate()
		glUseProgramObjectARB(Program)
	End Method


	Function DeActivate()
		glUseProgramObjectARB(0)
	EndFunction

	' Get the Uniform Variable Location from a ProgramObject
	Method GetUniLoc:Int(name:String)
		Return glGetUniformLocationARB(Program, Varptr name)
	End Method

	' Get the Attribute Variable Location from a ProgramObject
	Method GetAttribLoc:Int(name:String)
		Return glGetAttribLocationARB(Program, Varptr name)
	End Method
	

	' Set Uniform Variable Integer(s)
	Method setUI1(name:String,val:Int)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetUniformLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Uniform Var Location '"+name+"' in Method setUI1"
				Return
			End If
		?
		glUniform1iARB(loc,val)
		MemFree namePtr
	End Method

	Method setUI2(name:String,val1:Int,val2:Int)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetUniformLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Uniform Var Location '"+name+"' in Method setUI2"
				Return
			End If
		?
		glUniform2iARB(loc,val1,val2)
		MemFree namePtr
	End Method

	Method setUI3(name:String,val1:Int,val2:Int,val3:Int)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetUniformLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Uniform Var Location '"+name+"' in Method setUI3"
				Return
			End If
		?
		glUniform3iARB(loc,val1,val2,val3)
		MemFree namePtr
	End Method

	Method setUI4(name:String,val1:Int,val2:Int,val3:Int,val4:Int)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetUniformLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Uniform Var Location '"+name+"' in Method setUI4"
				Return
			End If
		?
		glUniform4iARB(loc,val1,val2,val3,val4)
		MemFree namePtr
		
	End Method

	Method getUI1:Int(name:String)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetUniformLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Uniform Var Location '"+name+"' in Method getUI1"
				Return 0
			End If
		?
		Local val:Int = 10
		glGetUniformivARB(Program, loc, Varptr(val))
		MemFree namePtr
		Return val
	End Method
	
	
	' Set Uniform Variable Float(s)
	Method setUF1(name:String,val:Float)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetUniformLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Uniform Var Location '"+name+"' in Method setUF1"
				Return
			End If
		?
		glUniform1fARB(loc,val)
		MemFree namePtr
	End Method

	Method setUF2(name:String,val1:Float,val2:Float)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetUniformLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print ">Error getting Uniform Var Location '"+name+"' in Method setUF2"
				Return
			End If
		?
		glUniform2fARB(loc,val1,val2)
		MemFree namePtr
	End Method

	Method setUF3(name:String,val1:Float,val2:Float,val3:Float)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetUniformLocationARB(Program, namePtr)

		?Debug
			If loc<0
				Print "Error getting Uniform Var Location '"+name+"' in Method setUF3"
				Return
			End If
		?
		glUniform3fARB(loc, val1, val2, val3)
		MemFree namePtr
	End Method

	Method setUF4(name:String,val1:Float,val2:Float,val3:Float,val4:Float)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetUniformLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Uniform Var Location '"+name+"' in Method setUF4"
				Return
			End If
		?
		glUniform4fARB(loc,val1,val2,val3,val4)
		MemFree namePtr
	End Method

	' Set Attribute Variable Short(s)
	Method setAS1(name:String, val1:Short)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetAttribLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Attribute Var Location '"+name+"' in Method setAS1"
				Return
			End If
		?
		glVertexAttrib1sARB(loc, val1)
		MemFree namePtr
	End Method

	Method setAS2(name:String, val1:Short, val2:Short)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetAttribLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Attribute Var Location '"+name+"' in Method setAS2"
				Return
			End If
		?
		glVertexAttrib2sARB(loc, val1, val2)
		MemFree namePtr
	End Method

	Method setAS3(name:String, val1:Short, val2:Short, val3:Short)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetAttribLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Attribute Var Location '"+name+"' in Method setAS3"
				Return
			End If
		?
		glVertexAttrib3sARB(loc, val1, val2, val3)
		MemFree namePtr
	End Method

	Method setAS4(name:String, val1:Short, val2:Short, val3:Short, val4:Short)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetAttribLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Attribute Var Location '"+name+"' in Method setAS4"
				Return
			End If
		?
		glVertexAttrib4sARB(loc, val1, val2, val3, val4)
		MemFree namePtr
	End Method
	
	' Set Attribute Variable Float(s)
	Method setAF1(name:String, val1:Float)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetAttribLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Attribute Var Location '"+name+"' in Method setAF1"
				Return
			End If
		?
		glVertexAttrib1fARB(loc, val1)
		MemFree namePtr
	End Method

	Method setAF2(name:String, val1:Float, val2:Float)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetAttribLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Attribute Var Location '"+name+"' in Method setAF2"
				Return
			End If
		?
		glVertexAttrib2fARB(loc, val1, val2)
		MemFree namePtr
	End Method

	Method setAF3(name:String, val1:Float, val2:Float, val3:Float)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetAttribLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Attribute Var Location '"+name+"' in Method setAS3"
				Return
			End If
		?
		glVertexAttrib3fARB(loc, val1, val2, val3)
		MemFree namePtr
	End Method

	Method setAF4(name:String, val1:Float, val2:Float, val3:Float, val4:Float)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetAttribLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Attribute Var Location '"+name+"' in Method setAF4"
				Return
			End If
		?
		glVertexAttrib4fARB(loc, val1, val2, val3, val4)
		MemFree namePtr
	End Method

	' Set Attribute Variable Double(s)
	Method setAD1(name:String, val1:Double)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetAttribLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Attribute Var Location '"+name+"' in Method setAD1"
				Return
			End If
		?
		glVertexAttrib1fARB(loc, val1)
		MemFree namePtr
	End Method

	Method setAD2(name:String, val1:Double, val2:Double)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetAttribLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Attribute Var Location '"+name+"' in Method setAD2"
				Return
			End If
		?
		glVertexAttrib2fARB(loc, val1, val2)
		MemFree namePtr
	End Method

	Method setAD3(name:String, val1:Double, val2:Double, val3:Double)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetAttribLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Attribute Var Location '"+name+"' in Method setAD3"
				Return
			End If
		?
		glVertexAttrib3fARB(loc, val1, val2, val3)
		MemFree namePtr
	End Method

	Method setAD4(name:String, val1:Double, val2:Double, val3:Double, val4:Double)
	    Local namePtr:Byte Ptr = name.ToCString()
		Local loc:Int = glGetAttribLocationARB(Program, namePtr)
		?Debug
			If loc<0
				Print "Error getting Attribute Var Location '"+name+"' in Method setAD4"
				Return
			End If
		?
		glVertexAttrib4fARB(loc, val1, val2, val3, val4)
		MemFree namePtr
	End Method
End Type

Global fps_frames:Int, fps_ms:Int, fps_anzeige:Int
Function GetFPS:Int()
	fps_frames = fps_frames+1
	If MilliSecs()-fps_ms >= 1000 Then
		fps_anzeige = fps_frames
		fps_frames   = 0
		fps_ms      = MilliSecs()
	EndIf
	Return fps_anzeige
End Function