' app
Type TDemo
	' lead time (look ahead to pretrigger instruments)
	Const LEAD_TIME:Int		= 500	' ms
	' ball ids
	Const BALL_GROUND:Int	= 0
	Const BALL_HIT5:Int		= 1
	Const BALL_HIT6:Int		= 2
	Const BALL_HIT4:Int		= 3
	Const BALL_BASS1:Int	= 4
	Const BALL_BASS2:Int	= 5
	Const BALL_MELC6:Int	= 6
	Const BALL_MELGIS5:Int	= 7
	Const BALL_MELG5:Int	= 8
	Const BALL_MELF5:Int	= 9
	Const BALL_MELC5:Int	= 10
	Const BALL_MELGIS4:Int	= 11
	Const BALL_MELF4:Int	= 12
	Const BALL_MELC4:Int	= 13
	Const BALL_CRASH1:Int	= 14
	Const BALL_CRASH2:Int	= 15
	Const BALL_KICK:Int		= 16
	' channel ids
	Const CHN_HIT5:Int		= 0
	Const CHN_HIT6:Int		= 1
	Const CHN_HIT4:Int		= 2
	Const CHN_BASS1:Int		= 3
	Const CHN_BASS2:Int		= 4
	Const CHN_MEL:Int		= 5
	Const CHN_CRASH1:Int	= 6
	Const CHN_CRASH2:Int	= 7
	Const CHN_KICK:Int		= 8
	Const CHN_PAD:Int		= 9
	Const CHN_MAIN:Int		= 15
	
	' files
	Global midi:TMidiFile
	Global song:TSound
	' midi controllers
	Global controllers:TController[] = New TController[16]
	' background
	Global background:TBackground
	' balls
	Global balls:TBall[] = New TBall[17]
	' time
	Global curTime:Int		' runtime
	Global playTime:Int		' play time of song
	Global dTime:Int		' delta time (per frame)
	Global timer:TTimer
	' environment
	Global lightRed:Double, lightGrn:Double, lightBlu:Double
	Global rotVel:Double	' peripheral speed in px/s
	
	' create and initialize app
	Function Init:TDemo()
		' initialize background
		background			= TBackground.Create([24, 27, 28, 36, 39, 40, 48])
		' initialize balls
		balls[BALL_GROUND]		= TBall.Create(0, 0, 120, 1)
		balls[BALL_HIT5]		= TBall.Create(84, 145, 24, -1)
		balls[BALL_HIT6]		= TBall.Create(66.5, 137, 16, -1)
		balls[BALL_HIT4]		= TBall.Create(106.5, 153, 32, -1)
		balls[BALL_BASS1]		= TBall.Create(140, 185, 64, -1)
		balls[BALL_BASS2]		= TBall.Create(40, 185, 64, -1)
		balls[BALL_MELC6]		= TBall.Create(-148, 129, 8, -1)
		balls[BALL_MELGIS5]		= TBall.Create(-138.5, 131, 10, -1)
		balls[BALL_MELG5]		= TBall.Create(-127, 133, 12, -1)
		balls[BALL_MELF5]		= TBall.Create(-113.5, 137, 16, -1)
		balls[BALL_MELC5]		= TBall.Create(-97.5, 141, 20, -1)
		balls[BALL_MELGIS4]		= TBall.Create(-78.5, 145, 24, -1)
		balls[BALL_MELF4]		= TBall.Create(-57, 149, 28, -1)
		balls[BALL_MELC4]		= TBall.Create(-33, 153, 32, -1)
		balls[BALL_CRASH1]		= TBall.Create(170, 145, 24, -1)
		balls[BALL_CRASH2]		= TBall.Create(-166, 158, 37, -1)
		balls[BALL_KICK]		= TBall.Create(0, 169, 48, -1)
		' initialize controllers
		controllers[CHN_HIT5]	= TControllerBall.Create(CHN_HIT5, BALL_HIT5)
		controllers[CHN_HIT6]	= TControllerBall.Create(CHN_HIT6, BALL_HIT6)
		controllers[CHN_HIT4]	= TControllerBall.Create(CHN_HIT4, BALL_HIT4)
		controllers[CHN_BASS1]	= TControllerBall.Create(CHN_BASS1, BALL_BASS1)
		controllers[CHN_BASS2]	= TControllerBall.Create(CHN_BASS2, BALL_BASS2)
		controllers[CHN_MEL]	= TControllerBallArray.Create(CHN_MEL, ..
		  [BALL_MELC6, BALL_MELGIS5, BALL_MELG5, BALL_MELF5, BALL_MELC5, BALL_MELGIS4, BALL_MELF4, BALL_MELC4],..
		  [        72,           68,         67,         65,         60,           56,         53,         48])
		controllers[CHN_CRASH1]	= TControllerBall.Create(CHN_CRASH1, BALL_CRASH1)
		controllers[CHN_CRASH2]	= TControllerBall.Create(CHN_CRASH2, BALL_CRASH2)
		controllers[CHN_KICK]	= TControllerBall.Create(CHN_KICK, BALL_KICK)
		controllers[CHN_PAD]	= TControllerPad.Create(CHN_PAD)
		controllers[CHN_MAIN]	= TControllerMain.Create(CHN_MAIN)
		' graphics
		SetGraphicsDriver GLMax2DDriver()
		Graphics(800,600)
		timer = CreateTimer(60)
		HideMouse()
		' sound
		Local drivers:String[] = ["FreeAudio DirectSound", "FreeAudio Multimedia", "FreeAudio", "Null"]
		For Local driver:String = EachIn drivers
			If AudioDriverExists(driver)
				SetAudioDriver(driver)
				Exit
			EndIf
		Next
		' load midi file
		midi = TMidiFile.LoadFile("incbin::_src/bcc58.mid")
		' load sound file
		song = LoadSound("incbin::_src/bcc58.ogg")
	End Function
	
	' run demo
	Function Run()
		' start demo
		Local stime:Int = MilliSecs()
		Local chn:TChannel = PlaySound(song)
		' main loop
		While Not (KeyHit(KEY_ESCAPE) Or AppTerminate())
			Cls
			
			Local otime:Int = curTime
			curTime = MilliSecs()
			playTime = curTime -stime
			dTime = curTime -otime
			' play midi
			Local trackEvents:TMidiTrackEvent[] = midi.Play(midi.TicksForPlaytime(playTime +LEAD_TIME))
			' include every track...
			For Local te:TMidiTrackEvent = EachIn trackEvents
				' ... and every event
				For Local e:TMidiEvent = EachIn te.events
					' select controller to send event to
					' channel event
					If e.eventType<255
						Local i:Int = e.channel
						If controllers[i] Then controllers[i].PushEvent(e)
					' meta event -> send to main channel
					Else
						controllers[15].PushEvent(e)
					EndIf
				Next
			Next
			
			' run controllers
			For Local c:TController = EachIn controllers
				If c Then c.Update()
			Next
			
			' draw background
			background.Draw()
			
			' draw balls
			TBall.DrawBalls()
			
			Flip 0
			WaitTimer(timer)
		Wend
	End Function
	
	' end demo
	Function ShutDown()
		End
	End Function
End Type

